*==============================================================================
* Program:           JumbleTron.PRG
* Author:            Rick Borup
* Date Written:      10/17/2005
* Copyright:         (c) 2005 Information Technology Associates
*                             All rights reserved.
* Compiler:          Visual FoxPro 09.00.0000.2412 for Windows
*
* Abstract:          A class to jumble a block of text by rearranging the 
*                    letters in each word while keeping the first and last
*                    letter the same. 
*
* Comments:          This class can be used to play around with the concept 
*                    that people can read jumbled text fairly easily, at least
*                    in English and other languages where words are similarly 
*                    structured.
*
*                    An arbitrary limit of 4K bytes in imposed on the incoming
*                    string of text.
*
* Changes:				10/20/2005 - v1.0.1
*                    - Refactored some code, added some comments, and other 
*                      minor tweaks.
*==============================================================================

DEFINE CLASS jumbleTron AS session
cJumbled = ""
nMaxSize = 4096	&& maximum size of incoming text block (arbitrary)

*--------------------------------------------------------------------
*   Jumble :: Init
*------------------------------------
*   Function: The class Init function
*       Pass: Nothing
*     Return: Logical
*   Comments: 
*--------------------------------------------------------------------
PROCEDURE Init() as Logical
=RAND( -1)	&& Initialize the RAND function.
ENDPROC && Init

*--------------------------------------------------------------------
*   Jumble :: GetJumbledText
*------------------------------------
*   Function: Jumble a block of text.
*       Pass: tcText - the original text
*     Return: Character
*   Comments: Only words of four characters or more get jumbled.
*--------------------------------------------------------------------
PROCEDURE GetJumbledText( tcText as String) as String
IF VARTYPE( tcText) <> "C"
	RETURN ""
ENDIF
IF LEN( tcText) > this.nMaXSize
	RETURN "Incoming text exceeds maximum allowable size of " + ;
			 TRANSFORM( this.nMaxSize)
ENDIF
this.cJumbled = ""
LOCAL lnLen, lni, lcWord, lcJumbledWord
lnLen = LEN( tcText)
lni = 1
DO WHILE lni <= lnLen
	*	Parse out a word.
	lcWord = ""
	DO WHILE lni <= lnLen AND ISALPHA( SUBSTR( tcText, lni, 1))
		lcWord = lcWord + SUBSTR( tcText, lni, 1)
		lni = lni + 1
	ENDDO
	*	If the word is more than 3 letters long, jumble it.
	lcJumbledWord = IIF( LEN( lcWord) > 3, this.GetJumbledWord( lcWord), lcWord)
	*	Tack it onto the end of the output string.
	this.cJumbled = this.cJumbled + lcJumbledWord
	*	Tack on any non-alpha characters up to the start of the next word
	*	or the end of the input string.
	DO WHILE lni <= lnLen AND !ISALPHA( SUBSTR( tcText, lni, 1))
		this.cJumbled = this.cJumbled + SUBSTR( tcText, lni, 1)
		lni = lni + 1
	ENDDO	
ENDDO
RETURN this.cJumbled
ENDPROC && GetJumbledText

*--------------------------------------------------------------------
*   Jumble :: GetJumbledWord
*------------------------------------
*   Function: Jumble a word by rearranging its letters
*       Pass: tcWord -the word to jumble
*     Return: Character
*   Comments: Keeps the first and last letter of the word the same
*             but jumbles the rest in random fashion.
*--------------------------------------------------------------------
PROTECTED FUNCTION GetJumbledWord( tcWord as String) as String
LOCAL lcJumbledWord
lcJumbledWord = IIF( VARTYPE( tcWord) = "C", ;
							LEFT( tcWord, 1) + ;
							this.JumbleString( SUBSTR( tcWord, 2, LEN( tcWord) - 2)) + ;
							RIGHT( tcWord, 1), ;
							"" ;
						 )
RETURN lcJumbledWord
ENDFUNC && GetJumbledWord

*--------------------------------------------------------------------
*   Jumble :: JumbleString
*------------------------------------
*   Function: Jumble a string of characters
*       Pass: tcString - the string to jumble
*     Return: Character
*   Comments: Create a cursor with one row for each letter plus a
*             random number for that letter. Then select from the
*             cursor in order by the random number and create the
*             return string from that.
*--------------------------------------------------------------------
PROTECTED FUNCTION JumbleString( tcString as String) as String
IF VARTYPE( tcString) <> "C"
	RETURN ""
ENDIF
IF USED( "csrJumble")
	USE IN csrJumble
ENDIF
LOCAL lni, lcJumbledString
CREATE CURSOR csrJumble ( cLetter c(1), nRand n(10,8))
FOR lni = 1 TO LEN( tcString)
	INSERT INTO csrJumble ( ;
			cLetter, ;
			nRand ;
		) ;
		VALUES ( ;
			SUBSTR( tcString, lni, 1), ;
			RAND() ;
		)
ENDFOR
SELECT cLetter ;
	FROM csrJumble ;
	INTO ARRAY laLetters ;
	ORDER BY nRand
lcJumbledString = ""
FOR lni = 1 TO ALEN( laLetters)
	lcJumbledString = lcJumbledString + laLetters[ lni]
ENDFOR
USE IN SELECT( "csrJumble")
RETURN lcJumbledString
ENDFUNC && JumbleString

ENDDEFINE && JumbleTron
